// Title:   BoxSelector.cpp
// Created: Thu Sep 25 15:16:54 2003
// Authors: Simon Heinzle, Tim Weyrich
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifdef _WIN32
	// turn off std::map warnings
	//# pragma warning(push)
	# pragma warning(disable:4786) // identifier truncated to 255 char
	# pragma warning(disable:4244) // conversion from .. to .. possible loss of data
#endif


#include "BoxSelector.h"
#include "Selector.h"

#include "../../../../Core/RendererManager/src/RendererManager.h"
#include "../../../../Core/Configuration/src/Configuration.h"
#include "../../../../Utilities/src/TrackBall.h"
#include "../../../../Renderers/OpenGLRenderer/src/OpenGLRenderer.h"
#include "../../../../DataTypes/src/MyDataTypes.h"

#include <qgl.h>
#include <math.h>
#include <stdlib.h> 

#ifndef NDEBUG
#  define  CHECK_FOR_GL_ERROR()													\
     do {																		\
       const char *str; int err;												\
       while ((err = glGetError ()) != GL_NO_ERROR) {							\
         switch (err)															\
           {																	\
           case GL_INVALID_ENUM:  str = "GL_INVALID_ENUM";  break;				\
           case GL_INVALID_VALUE:  str = "GL_INVALID_VALUE";  break;			\
           case GL_INVALID_OPERATION:  str = "GL_INVALID_OPERATION";  break;	\
           case GL_STACK_OVERFLOW:  str = "GL_STACK_OVERFLOW";  break;			\
           case GL_STACK_UNDERFLOW:  str = "GL_STACK_UNDERFLOW";  break;		\
           case GL_OUT_OF_MEMORY:  str = "GL_OUT_OF_MEMORY";  break;			\
           default:  str = "<unknown error>";									\
           }																	\
         qDebug ("OpenGL Error anywhere before %s:%d: %s\n",			   		\
     	     __FILE__, __LINE__, str); }										\
     } while(0)
#else
#  define  CHECK_FOR_GL_ERROR()  while(0)
#endif

// CONSTANTS
#include "NewOverload.h"

BoxSelector::BoxSelector() {
	
	_the_box[0] = Vector3D(-1.0,1.0,1.0);
	_the_box[1] = Vector3D(1.0,1.0,1.0);
	_the_box[2] = Vector3D(1.0,-1.0,1.0);
	_the_box[3] = Vector3D(-1.0,-1.0,1.0);
	_the_box[4] = Vector3D(-1.0,1.0,-1.0);
	_the_box[5] = Vector3D(1.0,1.0,-1.0);
	_the_box[6] = Vector3D(1.0,-1.0,-1.0);
	_the_box[7] = Vector3D(-1.0,-1.0,-1.0);
	
	_the_faces[0][0] = 0;
	_the_faces[0][1] = 1;
	_the_faces[0][2] = 2;
	_the_faces[0][3] = 3;

	_the_faces[1][0] = 1;
	_the_faces[1][1] = 5;
	_the_faces[1][2] = 6;
	_the_faces[1][3] = 2;

	_the_faces[2][0] = 4;
	_the_faces[2][1] = 7;
	_the_faces[2][2] = 6;
	_the_faces[2][3] = 5;

	_the_faces[3][0] = 0;
	_the_faces[3][1] = 3;
	_the_faces[3][2] = 7;
	_the_faces[3][3] = 4;

	_the_faces[4][0] = 0;
	_the_faces[4][1] = 4;
	_the_faces[4][2] = 5;
	_the_faces[4][3] = 1;
	
	_the_faces[5][0] = 3;
	_the_faces[5][1] = 2;
	_the_faces[5][2] = 6;
	_the_faces[5][3] = 7;
	
	_the_face_normals[0] = Vector3D(0.0,0.0,1.0);
	_the_face_normals[1] = Vector3D(1.0,0.0,0.0);
	_the_face_normals[2] = Vector3D(0.0,0.0,-1.0);
	_the_face_normals[3] = Vector3D(-1.0,0.0,0.0);
	_the_face_normals[4] = Vector3D(0.0,1.0,0.0);
	_the_face_normals[5] = Vector3D(0.0,-1.0,0.0);

	_the_x_lines[0][0] = 1;
	_the_x_lines[0][1] = 0;
	_the_x_lines[1][0] = 2;
	_the_x_lines[1][1] = 3;
	_the_x_lines[2][0] = 5;
	_the_x_lines[2][1] = 4;
	_the_x_lines[3][0] = 6;
	_the_x_lines[3][1] = 7;

	_the_y_lines[0][0] = 3;
	_the_y_lines[0][1] = 0;
	_the_y_lines[1][0] = 2;
	_the_y_lines[1][1] = 1;
	_the_y_lines[2][0] = 6;
	_the_y_lines[2][1] = 5;
	_the_y_lines[3][0] = 7;
	_the_y_lines[3][1] = 4;
	
	_the_z_lines[0][0] = 2;
	_the_z_lines[0][1] = 6;
	_the_z_lines[1][0] = 1;
	_the_z_lines[1][1] = 5;
	_the_z_lines[2][0] = 0;
	_the_z_lines[2][1] = 4;
	_the_z_lines[3][0] = 3;
	_the_z_lines[3][1] = 7;

	_selection = 0;

	_edgelength = Vector3D(100.0,100.0,100.0);
	_center[0] = 0.0;
	_center[1] = 0.0;
	_center[2] = 0.0;

	_x_normal = cQuaternion(1,0,0,0);
	_y_normal = cQuaternion(0,1,0,0);
	_z_normal = cQuaternion(0,0,1,0);

	for (int i=0; i<4; i++)
		for (int j=0; j<4;j++)
			_rotation_matrix[i][j] = 0.0;
	_rotation_matrix[0][0] = 1.0;
	_rotation_matrix[1][1] = 1.0;
	_rotation_matrix[2][2] = 1.0;
	_rotation_matrix[3][3] = 1.0;

	_picking_mode = false;
	_quadric_sphere = gluNewQuadric();
	_shape = SHAPE_BOX;
	_old_scene_set = false;
	_freeze_box = false;
	_align_mode = ALIGN_MEDIAN;
	_mouse_mode = MOUSE_ALIGN;
	
	_aa_box = false;
	_box_position_valid = false;

	_color_normal[0] = _color_normal_transparent[0] = 0.0f;
	_color_normal[1] = _color_normal_transparent[1] = 0.8f;
	_color_normal[2] = _color_normal_transparent[2] = 0.0f;
	_color_normal[3] = 1.0f; _color_normal_transparent[3] = 0.4f;

	_color_selected[0] = 0.8f;
	_color_selected[1] = 0.8f;
	_color_selected[2] = 0.8f;
	_color_selected[3] = 0.8f;


}

BoxSelector::BoxSelector(Vector3D init_center , Vector3D init_edge_length) {
	BoxSelector();
	_center = init_center;
	_edgelength = init_edge_length;
}

BoxSelector::~BoxSelector() {
	//nada
}

inline void BoxSelector::paintVertex(Vector3D mid, float size) {
	Vector3D temp;

	size /= 2.0;
	
	glBegin(GL_QUADS);
		for (int f=0; f<6; f++) {
			glNormal3fv(_the_face_normals[f].source());
			for (int v=0; v<4; v++) {
				temp = _the_box[_the_faces[f][v]]*size;
				temp += mid;
				glVertex3fv(temp.source());
			}
		}
	glEnd();
}
	

inline void BoxSelector::paintVertices(float size) {
	
	for (int v=0; v<8; v++) {
		glLoadName(VERT + v);
		
		if (_selection == VERT + v) {
			glMaterialfv(GL_FRONT,GL_DIFFUSE, _color_selected);
			glColor4fv(_color_selected);
		} else {
			glMaterialfv(GL_FRONT,GL_DIFFUSE, _color_normal);
			glColor4fv(_color_normal);
		}

		paintVertex(_the_box[v], size);
	}
	
};

inline void BoxSelector::paintLines(float thickness) {
	Vector3D line_start, line_end;
	Vector3D temp;
	int n;

	thickness /=2.0;
		
	// paint x lines
	if (_picking_mode)
		glLoadName(XROT);

	glMaterialfv(GL_FRONT,GL_DIFFUSE, _color_normal);
	glColor4fv(_color_normal);
	if (_selection == XROT) {
		glMaterialfv(GL_FRONT,GL_DIFFUSE, _color_selected);
		glColor4fv(_color_selected);
	}
	
	glBegin(GL_QUADS);
	for(n=0; n<4; n++) {
		line_start = _the_box[_the_x_lines[n][0]];
		line_end = _the_box[_the_x_lines[n][1]];
		// top
		glNormal3f(0.0,1.0,0.0);
		glVertex3f(line_start[0], line_start[1]+thickness, line_start[2]-thickness);
		glVertex3f(line_end[0], line_end[1]+thickness, line_end[2]-thickness);
		glVertex3f(line_end[0], line_end[1]+thickness, line_end[2]+thickness);
		glVertex3f(line_start[0], line_start[1]+thickness, line_start[2]+thickness);
			
		// bottom
		glNormal3f(0.0,-1.0,0.0);
		glVertex3f(line_start[0], line_start[1]-thickness, line_start[2]+thickness);
		glVertex3f(line_end[0], line_end[1]-thickness, line_end[2]+thickness);
		glVertex3f(line_end[0], line_end[1]-thickness, line_end[2]-thickness);
		glVertex3f(line_start[0], line_start[1]-thickness, line_start[2]-thickness);
				
		
		//front
		glNormal3f(0.0,0.0,1.0);
		glVertex3f(line_start[0], line_start[1]-thickness, line_start[2]+thickness);
		glVertex3f(line_start[0], line_start[1]+thickness, line_start[2]+thickness);
		glVertex3f(line_end[0], line_end[1]+thickness, line_end[2]+thickness);
		glVertex3f(line_end[0], line_end[1]-thickness, line_end[2]+thickness);
		
				
		//back
		glNormal3f(0.0,0.0,-1.0);
		glVertex3f(line_start[0], line_start[1]+thickness, line_start[2]-thickness);
		glVertex3f(line_start[0], line_start[1]-thickness, line_start[2]-thickness);
		glVertex3f(line_end[0], line_end[1]-thickness, line_end[2]-thickness);
		glVertex3f(line_end[0], line_end[1]+thickness, line_end[2]-thickness);
	}
	glEnd();
	
	// paint y lines
	if (_picking_mode)
		glLoadName(YROT);
	glMaterialfv(GL_FRONT,GL_DIFFUSE, _color_normal);
	glColor4fv(_color_normal);
	if (_selection == YROT) {
		glMaterialfv(GL_FRONT,GL_DIFFUSE, _color_selected);
		glColor4fv(_color_selected);
	}
	glBegin(GL_QUADS);
	for(n=0; n<4; n++) {
		line_start = _the_box[_the_y_lines[n][0]];
		line_end = _the_box[_the_y_lines[n][1]];
		// right
		glNormal3f(1.0,0.0,0.0);
		glVertex3f(line_start[0]+thickness, line_start[1], line_start[2]-thickness);
		glVertex3f(line_end[0]+thickness, line_end[1], line_end[2]-thickness);
		glVertex3f(line_end[0]+thickness, line_end[1], line_end[2]+thickness);
		glVertex3f(line_start[0]+thickness, line_start[1], line_start[2]+thickness);
		// left
		glNormal3f(-1.0,0.0,0.0);
		glVertex3f(line_end[0]-thickness, line_end[1], line_end[2]-thickness);
		glVertex3f(line_start[0]-thickness, line_start[1], line_start[2]-thickness);
		glVertex3f(line_start[0]-thickness, line_start[1], line_start[2]+thickness);
		glVertex3f(line_end[0]-thickness, line_end[1], line_end[2]+thickness);
		//front
		glNormal3f(0.0,0.0,1.0);
		glVertex3f(line_start[0]-thickness, line_start[1], line_start[2]+thickness);
		glVertex3f(line_start[0]+thickness, line_start[1], line_start[2]+thickness);
		glVertex3f(line_end[0]+thickness, line_end[1], line_end[2]+thickness);
		glVertex3f(line_end[0]-thickness, line_end[1], line_end[2]+thickness);
		
		//back
		glNormal3f(0.0,0.0,-1.0);
		glVertex3f(line_start[0]+thickness, line_start[1], line_start[2]-thickness);
		glVertex3f(line_start[0]-thickness, line_start[1], line_start[2]-thickness);
		glVertex3f(line_end[0]-thickness, line_end[1], line_end[2]-thickness);
		glVertex3f(line_end[0]+thickness, line_end[1], line_end[2]-thickness);
	}
	glEnd();
	

	// paint z lines
	if (_picking_mode)
			glLoadName(ZROT);
	glMaterialfv(GL_FRONT,GL_DIFFUSE, _color_normal);
	glColor4fv(_color_normal);
	if (_selection == ZROT) {
		glMaterialfv(GL_FRONT,GL_DIFFUSE, _color_selected);
		glColor4fv(_color_selected);
	}
	
	glBegin(GL_QUADS);
	for(n=0; n<4; n++) {
		line_start = _the_box[_the_z_lines[n][0]];
		line_end = _the_box[_the_z_lines[n][1]];
	
		// right
		glNormal3f(1.0,0.0,0.0);
		glVertex3f(line_start[0]+thickness, line_start[1]-thickness, line_start[2]);
		glVertex3f(line_end[0]+thickness, line_end[1]-thickness, line_end[2]);
		glVertex3f(line_end[0]+thickness, line_end[1]+thickness, line_end[2]);
		glVertex3f(line_start[0]+thickness, line_start[1]+thickness, line_start[2]);

		// left
		glNormal3f(-1.0,0.0,0.0);
		glVertex3f(line_end[0]-thickness, line_end[1]-thickness, line_end[2]);
		glVertex3f(line_start[0]-thickness, line_start[1]-thickness, line_start[2]);
		glVertex3f(line_start[0]-thickness, line_start[1]+thickness, line_start[2]);
		glVertex3f(line_end[0]-thickness, line_end[1]+thickness, line_end[2]);
	
		//top
		glNormal3f(0.0,1.0,0.0);
		glVertex3f(line_start[0]+thickness, line_start[1]+thickness, line_start[2]);
		glVertex3f(line_end[0]+thickness, line_end[1]+thickness, line_end[2]);
		glVertex3f(line_end[0]-thickness, line_end[1]+thickness, line_end[2]);
		glVertex3f(line_start[0]-thickness, line_start[1]+thickness, line_start[2]);
		
		
		//bottom
		glNormal3f(0.0,-1.0,0.0);
		glVertex3f(line_start[0]+thickness, line_start[1]-thickness, line_start[2]);
		glVertex3f(line_start[0]-thickness, line_start[1]-thickness, line_start[2]);
		glVertex3f(line_end[0]-thickness, line_end[1]-thickness, line_end[2]);
		glVertex3f(line_end[0]+thickness, line_end[1]-thickness, line_end[2]);
	}
	glEnd();
	
}
	
void BoxSelector::paintFaces() {
	Vector3D tmp;
		
	if (_selection == 0) { // no face selected
		// light back & front
		glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
		// enable blending
		glEnable(GL_BLEND);
		// disable z-buffer for translucent objects
		glDepthMask(GL_FALSE);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	
		for (int f=0; f<6; f++) {
			// for each face
			glLoadName(SIDE + f);
			glBegin(GL_QUADS);
			glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE, _color_normal_transparent);
			glColor4fv(_color_normal_transparent);
			glNormal3fv(_the_face_normals[f].source());
			for (int e=0; e<4; e++) {
				tmp = _the_box[_the_faces[f][e]];
				glVertex3fv(tmp.source());
			}
			glEnd();
		}

	
		// disable blending & enable z-buffering
		glDepthMask(GL_TRUE);
		glDisable(GL_BLEND);
		glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
	} else {
		glBegin(GL_QUADS);
		glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE, _color_selected);
		glColor4fv(_color_selected);
		for (int f=0; f<6; f++) 
			if (_selection - SIDE == f) {
				glNormal3fv(_the_face_normals[f].source());
				for (int e=0; e<4; e++) {
					tmp = _the_box[_the_faces[f][e]] ;
					glVertex3fv(tmp.source());
				}
			}
		glEnd();
	}
	
}

void BoxSelector::paintSphere() {
	glPushMatrix();
	// light back & front
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
	// enable blending
	glEnable(GL_BLEND);
	// disable z-buffer for translucent objects
	glDepthMask(GL_FALSE);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);

	glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE, _color_normal_transparent);
	glColor4fv(_color_normal_transparent);
	gluSphere(_quadric_sphere, 1.f, 20,20);

	// disable blending & enable z-buffering
	glDepthMask(GL_TRUE);
	glDisable(GL_BLEND);
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
	glPopMatrix();
}


void BoxSelector::paintBox() {
	Vector3D temp(0.0,0.0,0.0);
	
	getAlignRect();
	
	if (!_box_position_valid)
		return;

	glColor3f(1.0,0.0,0.0);

	glPushMatrix();
	glEnable(GL_NORMALIZE);
	glTranslatef(_center[0], _center[1], _center[2]);
	glMultMatrixf(&_rotation_matrix[0][0]);
	glScalef(_edgelength[0]/2.f,_edgelength[1]/2.f,_edgelength[2]/2.f);
	
	// draw vertices
	
	glMaterialfv(GL_FRONT,GL_DIFFUSE, _color_normal);
	glColor4fv(_color_normal);
	paintVertices((float)(1.0/15.0));
	
	// draw edges
	glMaterialfv(GL_FRONT,GL_DIFFUSE, _color_normal);
	glColor4fv(_color_normal);
	paintLines((float)(1.0/40.0));
	
	// draw faces
	// in sorted waw!
	glMaterialfv(GL_FRONT,GL_DIFFUSE, _color_normal_transparent);
	glColor4fv(_color_normal_transparent);
	
	glEnable(GL_CULL_FACE);
    glCullFace(GL_FRONT);
	switch (_shape) {
		case SHAPE_BOX:
			paintFaces();
			glCullFace(GL_BACK);
			paintFaces();//_edgelength
			break;
		case SHAPE_SPHERE:
			if (_selection!=0)
				paintFaces();
			paintSphere();
			glCullFace(GL_BACK);
			paintSphere();
			break;
	}
	
	glDisable(GL_CULL_FACE);
	glDisable(GL_NORMALIZE);
	glPopMatrix();

}



void BoxSelector::paintBoxSelect() {
	Vector3D temp(0.0,0.0,0.0);
	RendererManager *renderManager;
	OpenGLRenderer *renderGL;
	MyDataTypes::TransformationMatrix16f sceneViewMatrix;

	if (!_box_position_valid)
		return;

	renderManager = RendererManager::getInstance();
	renderGL = (OpenGLRenderer*) renderManager->getPreviewRenderer();
	
	renderGL->getSceneView(sceneViewMatrix);
	
	glPushMatrix();
	glLoadIdentity();
	glMultMatrixf(&sceneViewMatrix[0]);

	glTranslatef(_center[0], _center[1], _center[2]);
	glMultMatrixf(&_rotation_matrix[0][0]);
	glScalef(_edgelength[0]/2.f,_edgelength[1]/2.f,_edgelength[2]/2.f);

	glInitNames();
	glPushName(0);

	// draw vertices
	paintVertices(1.f/15.f);

	// draw edges
	paintLines(1.f/40.f);

	// draw faces
	paintFaces();

	glPopName();
	glPopMatrix();
}


int BoxSelector::getMouseSelection(int x, int y) {
	int objectsFound = 0;	
	int	viewportCoords[4] = {0};
	QSize viewportsize;
	// This will hold the ID's of the objects we click on.
	// 64/8 = 8 could be found!
	unsigned int selectBuffer[64] = {0};	
	

	_picking_mode = true;

	RendererManager *renderManager;
	OpenGLRenderer *renderGL;
	MyDataTypes::ViewFrustum frustum;

	glSelectBuffer(64, selectBuffer);			// register buffer
	glGetIntegerv(GL_VIEWPORT, viewportCoords);	// get view port coordinates
	glRenderMode(GL_SELECT);
	glMatrixMode(GL_PROJECTION);

	glPushMatrix();
		
		glLoadIdentity();
	
		renderManager = RendererManager::getInstance();
		renderGL = (OpenGLRenderer*) renderManager->getPreviewRenderer();
		viewportsize = renderGL->getViewPortSize();
		
		viewportCoords[0] = 0;
		viewportCoords[1] = 0;
		viewportCoords[2] = viewportsize.width();
		viewportCoords[3] = viewportsize.height();
		
		// gluPickMatrix allows us to create a projection matrix that is around our
		// cursor. First 2 parameters are the X and Y position, then the next 2
		// are the width and height of the region from the starting point.  The last parameter is
		// the view port coordinates.
		gluPickMatrix(x, viewportCoords[3] - y, 1, 1, viewportCoords);

		frustum = renderGL->getViewFrustum();

		gluPerspective (frustum.fieldOfView, frustum.aspectRatio,
		            frustum.nearPlane,   frustum.farPlane);
		//gluPerspective(45.0f, (GLfloat) (viewportCoords[2]-viewportCoords[0]) / (GLfloat) (viewportCoords[3]-viewportCoords[1]), 0.1f, 100.0f);
		glMatrixMode(GL_MODELVIEW);

		paintBoxSelect();
		
		glMatrixMode(GL_PROJECTION);	
		objectsFound = glRenderMode(GL_RENDER);
	glPopMatrix();	
	glMatrixMode(GL_MODELVIEW);

	//objectsFound = 1;  // TW
	qDebug("hits: %d\n", objectsFound);
	_selection = 0;
	if (objectsFound > 0) {
		// The only ones we care about are the minimum depth value (the second value) and
		// the object ID that was passed into glLoadName() (This is the fourth value).
		unsigned int lowestDepth = selectBuffer[1];
		int selectedObject = selectBuffer[3];

		for(int i = 1; i < objectsFound; i++) 
			if(selectBuffer[(i * 4) + 1] < lowestDepth){
				lowestDepth = selectBuffer[(i * 4) + 1];
				selectedObject = selectBuffer[(i * 4) + 3];
			}
		_selection = selectedObject;
		qDebug("found object is %d\n",_selection);
		_picking_mode = false;
		return selectedObject;
	}

	_picking_mode = false;
	_selection = 0;
	return 0;
}

void BoxSelector::resize(float additional_length) {
	_edgelength += Vector3D(additional_length,additional_length,additional_length);
}

void BoxSelector::resize(Vector3D additional_length) {
	_edgelength += additional_length;
}

void BoxSelector::translate(Vector3D translation) {
	_center += translation;
}

void BoxSelector::rotate(float angle, int axis) {
	float rot_matrix[4][4];
	float res_matrix[4][4];

	switch(axis) {
		case XROT:
			// rotate the other axis - normals
			_y_normal.rotateDEG(angle,_x_normal);
			_z_normal.rotateDEG(angle,_x_normal);
			// construct new rotation matrix for OpenGL
			cQuaternion::getRotationMatrixDEG(angle,_x_normal,rot_matrix);
			break;
		case YROT:
			_x_normal.rotateDEG(angle,_y_normal);
			_z_normal.rotateDEG(angle,_y_normal);
			cQuaternion::getRotationMatrixDEG(angle,_y_normal,rot_matrix);
			break;
		case ZROT:
			_x_normal.rotateDEG(angle,_z_normal);
			_y_normal.rotateDEG(angle,_z_normal);
			cQuaternion::getRotationMatrixDEG(angle,_z_normal,rot_matrix);
			break;
	}
	// multiply newly constructed rotation matrix with old rot. matrix
	MtrMult4x4f(&rot_matrix[0][0],&_rotation_matrix[0][0],&res_matrix[0][0]);
	// set the new matrix
	memcpy(&_rotation_matrix[0][0], &res_matrix[0][0], 16*sizeof(float));
	// set _aa_box flag to false to select right box
	_aa_box = false;
}

	
void BoxSelector::reset(void) {
	_selection = 0;

	_edgelength = Vector3D(100.0,100.0,100.0);
	_center[0] = 0.0;
	_center[1] = 0.0;
	_center[2] = 0.0;

	int i;
	
	for (i=0; i<4; i++) {
		_x_normal[i] = 0.0;
		_y_normal[i] = 0.0;
		_z_normal[i] = 0.0;
	}
	
	_x_normal[0] = 1.0;
	_y_normal[1] = 1.0;
	_z_normal[2] = 1.0;
	
	for (i=0; i<4; i++)
		for (int j=0; j<4;j++)
			_rotation_matrix[i][j] = 0.0;
	_rotation_matrix[0][0] = 1.0;
	_rotation_matrix[1][1] = 1.0;
	_rotation_matrix[2][2] = 1.0;
	_rotation_matrix[3][3] = 1.0;
}


cQuaternion BoxSelector::getX(void) {
	return _x_normal;
}

cQuaternion BoxSelector::getY(void) {
	return _y_normal;
}

cQuaternion BoxSelector::getZ(void) {
	return _z_normal;
}

void BoxSelector::setSelection(int sel) { 
	_selection = sel;
}

int BoxSelector::getSelection(void) { 
	return _selection; 
}

void BoxSelector::deselect(void) { 
	_picking_mode = false; 
	_selection = 0; 
}

Vector3D BoxSelector::getEdgeLength(void) { 
	return _edgelength; 
}

Vector3D BoxSelector::getCenter(void) { 
	return _center; 
}

void BoxSelector::setCenter(Vector3D newcen) { 
	_center = newcen; 
}

void BoxSelector::setEdgeLength(Vector3D newlen) { 
	_edgelength = newlen; 
}

void BoxSelector::translateAlongFaceNormals(Vector3D translation) {
	Vector3D final_translation(0.0,0.0,0.0);

	final_translation += Vector3D(_x_normal[0],_x_normal[1], _x_normal[2]) * translation[0];
	final_translation += Vector3D(_y_normal[0],_y_normal[1], _y_normal[2]) * translation[1];
	final_translation += Vector3D(_z_normal[0],_z_normal[1], _z_normal[2]) * translation[2];

	_center += final_translation;
}

void BoxSelector::selectBox(int selection_index) {
	Selector *selector = Selector::getInstance();
	
	if (_shape == SHAPE_SPHERE) {
		selector->selectBox(this,selection_index,false);
	}
	if (_shape == SHAPE_BOX) {
		if (_aa_box) {
			// calculate upper&lowerbound
			Vector3D lowerbound(-fabs(_edgelength[0]/2.f),-fabs(_edgelength[1]/2.f),-fabs(_edgelength[2]/2.f));
			Vector3D upperbound(fabs(_edgelength[0]/2.f),fabs(_edgelength[1]/2.f),fabs(_edgelength[2]/2.f));
			lowerbound += _center;
			upperbound += _center;

			selector->selectAxisAlignedBox(lowerbound,upperbound, selection_index, false);
		} else {
			selector->selectBox(this,selection_index,false);
		}
	}
}

void BoxSelector::handleMousePress(int x, int y, int button) {
	_old_mouse_x = x;
	_old_mouse_y = y;
	
	
	if ( !RendererManager::getInstance()->isPreviewRendererEnabled() )
		return;	

	if (button&CTRLBUTTON) {
		// ignore
	} else if (button&LEFTBUTTON) {
		// select
		if (_mouse_mode == MOUSE_ALIGN) {
			if (_freeze_box)
				return;

			alignToScreenPos();
			
			if (!_box_position_valid)
				return;

			selectBox(_selection_index);
		}
		if (_mouse_mode == MOUSE_FREEMOVE) {
			if (_freeze_box)
				return;
			selectBox(_selection_index);
		}
		if (_mouse_mode == MOUSE_PICKING) {
			getMouseSelection(x,y);
		}
	}
	
	
	return;
}

void BoxSelector::handleMouseRelease(int x, int y, int button) {
	if ( !RendererManager::getInstance()->isPreviewRendererEnabled() )
		return;
	_old_mouse_x = x;
	_old_mouse_y = y;
	//mapPosToMouse();
	_selection = 0;
}

void BoxSelector::handleMouseMovementMidpoint(int x, int y, int button) {
	// ctrl -> do navigation
	if (button&CTRLBUTTON) {
		
		/**
		 * here you find copy-paste code from the navigation tool
		 * if ctrl is pressed mouse movement will do the same as in the navigation tool
		 */

		float normalizedMousePositions[4];
		float rotationVector[4];

		QSize viewPortSize;
		Scene *scene;
		Vector3D transformation, finalTransformation;

		viewPortSize = Configuration::getInstance()->getViewPortSize();
		scene = Scene::getInstance();
		
		
		// left mouse button -> rotate scene
		if (button&LEFTBUTTON) {
			// normalize mouse positions into rectangle, [-1.0, 1.0] x [-1.0, 1.0]
			normalizedMousePositions[0] = (float)_old_mouse_x / (float)viewPortSize.width()  * 2.0f - 1.0f;
			normalizedMousePositions[1] = (float)_old_mouse_y / (float)viewPortSize.height() * 2.0f - 1.0f;
			normalizedMousePositions[2] = (float)x / (float)viewPortSize.width()  * 2.0f - 1.0f;
			normalizedMousePositions[3] = (float)y / (float)viewPortSize.height() * 2.0f - 1.0f;
		
			if (TrackBall::calculateRotationMatrix (normalizedMousePositions, rotationVector) == true) {
				scene->rotate (rotationVector[0] * PI_180, rotationVector[1], rotationVector[2], rotationVector[3]);
			}
			
		}
			
		// middle mouse button -> translate perpendicular to viewing plane (into  -z)
		else if (button&MIDBUTTON) {
			scene->translate (0.0f, 0.0f, -2.0f * (y - _old_mouse_y));
		}

		// right mouse button -> translate  parallel to viewing plane (x,y)
		else if (button&RIGHTBUTTON) {
			scene->translate (x - _old_mouse_x, _old_mouse_y - y, 0.0f);	
		} 
		
		// otherwise align to screen pos
		else if ( RendererManager::getInstance()->isPreviewRendererEnabled())
			if (!_freeze_box && _mouse_mode == MOUSE_ALIGN)
				alignToScreenPos();
		
	}
	// no special key pressed -> move box
	else if (!_freeze_box && _mouse_mode == MOUSE_ALIGN && RendererManager::getInstance()->isPreviewRendererEnabled()) {
		
		/**
		 * moving the box
		 */

		// right button -> resize
		if (button&RIGHTBUTTON) { 
			resizeAfterMouseMovement(_old_mouse_y-y);
		} 
		// align box to screen pos
		else {
			_box_position_valid = false;
			alignToScreenPos();
			// left button -> mark 
			if (_box_position_valid && button&LEFTBUTTON)
				selectBox(_selection_index);
		}
	} 
	// picking mode enabled
	else if (_mouse_mode == MOUSE_PICKING && RendererManager::getInstance()->isPreviewRendererEnabled()) {
		
		/**
		 * picking the box
		 */

		// vertex selected -> do nothing
		if (_selection>=VERT && _selection<VERT+8) {
			GLdouble screenx,screeny,screenz;
			GLdouble objx, objy, objz;
			GLdouble modelMatrix[16], projMatrix[16];
			GLint viewport[4];

			// move parallel to screen
			Vector3D vertex, vertex_transformed;
			Vector3D vert_to_mid;
			int vertex_no;

			vertex_no = _selection - VERT;
			vertex = _the_box[vertex_no];
			
			// get vector pointing from vertex to midpoint
			MtrMultVector3D(&_rotation_matrix[0][0], vertex, &vertex_transformed);
			vertex_transformed += _center;
			
			// get Opengl matrices & viewport
			glGetDoublev(GL_MODELVIEW_MATRIX, modelMatrix);			
			glGetDoublev(GL_PROJECTION_MATRIX, projMatrix);
			glGetIntegerv(GL_VIEWPORT, viewport);

			// get z value of our edge
			gluProject(vertex_transformed[0], vertex_transformed[1], vertex_transformed[2], modelMatrix, projMatrix, viewport, &screenx, &screeny, &screenz);
			// unproject our mouse pos into screen
			gluUnProject(x, viewport[3]-y-1, screenz, modelMatrix, projMatrix, viewport, &objx, &objy, &objz);

			
			_center = _center + (Vector3D(objx,objy,objz) - vertex_transformed);

			qDebug("%f|%f|%f   %f|%f|%f", vertex_transformed[0],vertex_transformed[1],vertex_transformed[2],objx,objy,objz);
		 }
		// side selected -> resize along face normal
		if (_selection>=SIDE && _selection<SIDE+6) {
			GLdouble objx,objy,objz;
			GLdouble screenx, screeny, screenz;
			GLdouble modelMatrix[16], projMatrix[16];
			GLint viewport[4];
			Vector3D normal, transformed_normal;
			Vector3D vector;
			float resizement;
			
			// transform normal
			normal = _the_face_normals[_selection - SIDE];
			MtrMultVector3DHom(&_rotation_matrix[0][0], normal, &transformed_normal);
			// get OpenGL matrices
			glGetDoublev(GL_MODELVIEW_MATRIX, modelMatrix);			
			glGetDoublev(GL_PROJECTION_MATRIX, projMatrix);
			glGetIntegerv(GL_VIEWPORT, viewport);
			// get vector (in world space) pointing in mouse movement direction, use z-value of box as hint
			gluProject(_center[0], _center[1], _center[2], modelMatrix, projMatrix, viewport, &screenx, &screeny, &screenz);
			gluUnProject(x, viewport[3] - y - 1, screenz, modelMatrix, projMatrix, viewport, &objx, &objy, &objz);
			vector = Vector3D(objx,objy,objz);
			gluUnProject(_old_mouse_x,viewport[3] - _old_mouse_y - 1, screenz, modelMatrix, projMatrix, viewport, &objx, &objy, &objz);
			vector -= Vector3D(objx,objy,objz);

			// project vector onto normal
			//vector = Vector3D::dotProduct(vector, transformed_normal) * transformed_normal;
			resizement = Vector3D::dotProduct(vector, transformed_normal);
			// resize
			normal[0] = fabs(normal[0]);	
			normal[1] = fabs(normal[1]);
			normal[2] = fabs(normal[2]);
			vector = resizement * normal;
			resize(vector);
		}
		// edge selected -> rotate along this edge
		if (_selection>=XROT && _selection<=ZROT) {
			// get 2 points on screen from axis
			GLdouble screenz;
			GLdouble modelMatrix[16], projMatrix[16];
			GLint viewport[4];
			Vector3D point;
			Vector3D axis, transformed_axis;
			GLdouble p1[2],p2[2];
			Vector2D screen_axis,mouse;
			float rotation;

			// get OpenGL matrices & viewport
			glGetDoublev(GL_MODELVIEW_MATRIX, modelMatrix);			
			glGetDoublev(GL_PROJECTION_MATRIX, projMatrix);
			glGetIntegerv(GL_VIEWPORT, viewport);

			// get axis vector
			switch(_selection) {
				case XROT: 
					axis = Vector3D(1.0,0.0,0.0);
					break;
				case YROT:
					axis = Vector3D(0.0,1.0,0.0);
					break;
				case ZROT:
					axis = Vector3D(0.0,0.0,1.0);
					break;
			}
			
			// transform axis vector
			MtrMultVector3D(&_rotation_matrix[0][0], axis, &transformed_axis);
			// get two points
			point = _center + transformed_axis;

			// project two points into screen
			gluProject(_center[0],_center[1],_center[2],modelMatrix,projMatrix,viewport, &p1[0],&p1[1],&screenz);
			gluProject(point[0],point[1],point[2],modelMatrix,projMatrix,viewport,&p2[0],&p2[1],&screenz);
			
			// get axis on screen.
			screen_axis[0] = p2[0] - p1[0];
			screen_axis[1] = p2[1] - p1[1];
			screen_axis.normalize();
			float tmp = screen_axis[0];
			screen_axis[0] = screen_axis[1];
			screen_axis[1] = -tmp;
			// get mouse movement
			mouse[0] = x - _old_mouse_x;
			mouse[1] = _old_mouse_y - y;

			// get rotation
			rotation = Vector2D::dotProduct(screen_axis,mouse);
			rotate(rotation,_selection);
		}
	}
	// else navigate parallel & perpendicular to screen
	else if (!_freeze_box && _mouse_mode == MOUSE_FREEMOVE && RendererManager::getInstance()->isPreviewRendererEnabled()) {
 		
		/**
		 * navigate free
		 */

		GLdouble objx,objy,objz;
		GLdouble screenx, screeny, screenz;
		GLdouble transz;	

		GLdouble modelMatrix[16], projMatrix[16];
		GLint viewport[4];

		glGetDoublev(GL_MODELVIEW_MATRIX, modelMatrix);			
		glGetDoublev(GL_PROJECTION_MATRIX, projMatrix);
		glGetIntegerv(GL_VIEWPORT, viewport);
		
		_box_position_valid = true;

		// mid button -> move on plane orthogonal to screen
		if (button&MIDBUTTON) { 
			// first project midpoint to get screen-coordinates
			gluProject(_center[0],_center[1],_center[2],modelMatrix, projMatrix, viewport, &screenx, &screeny, &screenz);  
			// unproject		
			transz = (_old_mouse_y-y)  * 0.00001f;
			gluUnProject(screenx,screeny,screenz-transz,modelMatrix, projMatrix, viewport, &objx, &objy, &objz);
			
			_center = Vector3D(objx,objy,objz);	
		} 
		// right button -> resize the cube
		else if (button&RIGHTBUTTON) { 
			resizeAfterMouseMovement(_old_mouse_y-y);
		} 
		// no button -> move on plane parallel to screen
		else { 
			// first project midpoint to get screenz
			gluProject(_center[0],_center[1],_center[2],modelMatrix, projMatrix, viewport, &screenx, &screeny, &screenz);  
			// unproject
			gluUnProject(x,viewport[3] - y - 1,screenz,modelMatrix, projMatrix, viewport, &objx, &objy, &objz);
			// set new center
			_center = Vector3D(objx, objy, objz);
		}
		// select if left button pressed
		if (button&LEFTBUTTON)
				selectBox(_selection_index);
	}

	
	// update mouse positions
	_old_mouse_x = x;
	_old_mouse_y = y;

	return;
}

void BoxSelector::handleKeyPressed(int key) {

}
void BoxSelector::handleKeyReleased(int key) {
	switch (key) {
		case Qt::Key_S:
			alignToNearest();
			break;
		case Qt::Key_A:
			alignToScreenPos();
			break;
		case Qt::Key_CapsLock:
			_freeze_box = !_freeze_box;
			break;
	}
}


void BoxSelector::alignToNearest() {
	Selector *selector = Selector::getInstance();
	KdTree *kdtree = selector->returnKdTree();
	kdtree->setNOfNeighbours(10);
	kdtree->queryPosition(_center);
	_center = kdtree->getNeighbourPosition(1);
}

void BoxSelector::mapPosToMouse(void) {
	GLdouble screenx, screeny, screenz;
	GLdouble modelMatrix[16], projMatrix[16];
	GLint viewport[4];
	
	// render frame for actual frame
	
	glGetDoublev(GL_MODELVIEW_MATRIX, modelMatrix);			
	glGetDoublev(GL_PROJECTION_MATRIX, projMatrix);
	glGetIntegerv(GL_VIEWPORT, viewport);

	gluProject(_center[0],_center[1],_center[2],modelMatrix, projMatrix, viewport, &screenx, &screeny, &screenz);  
	// adjust y coordinate
	screeny = viewport[3] - screeny - 1;

	// current screenx/y is relative to the renderer Widget, map it to absolute screen
	QWidget *widget = RendererManager::getInstance()->getPreviewRenderer()->getRendererWidget();
	QPoint pos = widget->mapToGlobal(QPoint(screenx,screeny));
	QCursor::setPos(pos.x(), pos.y()); 
	_old_mouse_x = pos.x();
	_old_mouse_y = pos.y();
}

void BoxSelector::setShape(int new_shape) {
	_shape = new_shape;
	RendererManager::getInstance()->getPreviewRenderer()->renderFrame();
}


int BoxSelector::getShape(void) { 
	return _shape; 
}

bool BoxSelector::isFreezed() { 
	return _freeze_box; 
}

void BoxSelector::setMouseMode(int mode) {
	if (mode == MOUSE_PICKING)
		_freeze_box = true;
	else
		_freeze_box = false;
	_mouse_mode = mode; 

}

gsl_matrix* BoxSelector::getTransformationMatrixGSL() {
	gsl_matrix *rotation_matrix;
	gsl_matrix *translation_matrix;
	gsl_matrix *scale_matrix;
	gsl_matrix *result_matrix;
	gsl_matrix *tmp_matrix;

	// set up rotation matrix
	rotation_matrix = gsl_matrix_alloc(4,4);
	for (int i=0; i<4; i++) 
		for (int j=0; j<4; j++) 
			gsl_matrix_set(rotation_matrix, i, j, _rotation_matrix[j][i]);

	// set up scale matrix
	scale_matrix = gsl_matrix_alloc(4,4); 
	gsl_matrix_set_zero(scale_matrix);
	gsl_matrix_set(scale_matrix,0,0,_edgelength[0]/2.f);
	gsl_matrix_set(scale_matrix,1,1,_edgelength[1]/2.f);
	gsl_matrix_set(scale_matrix,2,2,_edgelength[2]/2.f);
	gsl_matrix_set(scale_matrix,3,3,1);

	// set up translation matrix
	translation_matrix = gsl_matrix_alloc(4,4);
	gsl_matrix_set_identity(translation_matrix);
	gsl_matrix_set(translation_matrix,0,3,_center[0]);
	gsl_matrix_set(translation_matrix,1,3,_center[1]);
	gsl_matrix_set(translation_matrix,2,3,_center[2]);
	
	// compute resulting transformation matrix
	tmp_matrix = gsl_matrix_alloc(4,4);
	result_matrix = gsl_matrix_alloc(4,4);

	gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, translation_matrix, rotation_matrix, 0.0, tmp_matrix);	
	gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, tmp_matrix, scale_matrix, 0.0, result_matrix);	

	// deallocate matrices:
	gsl_matrix_free(tmp_matrix);
	gsl_matrix_free(translation_matrix);
	gsl_matrix_free(scale_matrix);
	gsl_matrix_free(rotation_matrix);
	
	return result_matrix;
}

int float_compare(const void *a, const void *b) {
	float *x,*y;
	x = (float*)a;
	y = (float*)b;
	if ( *x == *y ) 
		return 0;
	else if ( *x < *y ) 
		return -1;
	else 
		return 1;
}

void BoxSelector::alignToScreenPos() {
	QWidget *widget;
	GLdouble screenx, screeny,screenz;
	GLdouble objx, objy, objz;
	GLdouble modelMatrix[16], projMatrix[16];
	GLint viewport[4];

	glGetDoublev(GL_MODELVIEW_MATRIX, modelMatrix);			
	glGetDoublev(GL_PROJECTION_MATRIX, projMatrix);
	glGetIntegerv(GL_VIEWPORT, viewport);
	widget = RendererManager::getInstance()->getPreviewRenderer()->getRendererWidget();

	screenx = widget->mapFromGlobal(QCursor::pos()).x();
	screeny = viewport[3] - widget->mapFromGlobal(QCursor::pos()).y() - 1;
	
	switch (_align_mode) {
		case ALIGN_LOWEST:
			screenz = alignUseLowest();
			break;
		case ALIGN_MEDIUM:
			screenz = alignUseMedium();
			break;
		default:
			screenz = alignUseMedian();
	}
			
	gluUnProject(screenx,screeny,screenz,modelMatrix, projMatrix, viewport, &objx, &objy, &objz);  
	
	if (screenz == 1.0f) 
		_box_position_valid = false;
	else {
		_center = Vector3D(objx,objy,objz);
		_box_position_valid = true;
	}
	
}


float BoxSelector::alignUseMedian() {
	float array[25];
	int i_start;
	int i_end;
	int i;
	int pos;

	// use median
	qsort(_align_rect,25,sizeof(float),float_compare);
#if 0	
	// debug output
	qDebug("-- new align");
	qDebug("%f|%f|%f|%f|%f|%f|%f|%f|%f|%f|%f|%f|%f|%f|%f|%f|%f|%f|%f|%f|%f|%f|%f|%f|%f",
		_align_rect[0],_align_rect[1],_align_rect[2],_align_rect[3],_align_rect[4],_align_rect[5],_align_rect[6],_align_rect[7],_align_rect[8],
		_align_rect[9],_align_rect[10],_align_rect[11],_align_rect[12],_align_rect[13],_align_rect[14],_align_rect[15],_align_rect[16],
		_align_rect[17],_align_rect[18],_align_rect[19],_align_rect[20],_align_rect[21],_align_rect[22],_align_rect[23],_align_rect[24]);
#endif
	// remove 0.0 and 1.0 values from array
	for (i=0,pos=0; i<25; i++)
		if ((_align_rect[i] != 1.0)&&(_align_rect[i] != 0.0)) {
			array[pos] = _align_rect[i];
			pos++;
		}

	// we only want to use the 5 middle values (or less)

	if (pos<=5) {
		i_start = 0;
		i_end = pos;
	} else {
		i_start = floor(((double)pos) / 2.0 - 2.5);
		i_end = ceil(((double)pos) / 2.0 + 2.5);
	}
	
	if (i_start == i_end)
		return 1.0;

	i = (i_start + i_end) / 2.;
	
	// return array[i];
	return array[i];
}

// following use only the middle 5 values
float BoxSelector::alignUseMedium() {
	float array[25];
	float medium;
	float count;
	int i_start;
	int i_end;
	int i;
	int pos;


	// use median
	qsort(_align_rect,25,sizeof(float),float_compare);
	
	// remove 0.0 and 1.0 values from array
	for (i=0,pos=0; i<25; i++)
		if ((_align_rect[i] != 1.0)&&(_align_rect[i] != 0.0)) {
			array[pos] = _align_rect[i];
			pos++;
		}

	// we only want to use the 5 middle values (or less)

	if (pos<=5) {
		i_start = 0;
		i_end = pos;
	} else {
		i_start = floor(((double)pos) / 2.0 - 2.5);
		i_end = ceil(((double)pos) / 2.0 + 2.5);
	}
	
	if (i_start == i_end)
		return 1.0;
	
	count = 0;
	medium = 0.f;
	// use medium
	for (i=i_start; i<i_end; i++) {
		medium += array[i];
		count++;
	}

	// return medium
	return medium / count;
}

float BoxSelector::alignUseLowest() {
	float lowest;
	int i;

	lowest = 1.0;

	for (i=0; i<25; i++)
		if ((_align_rect[i] != 0)&&(_align_rect[i] < lowest))
			lowest = _align_rect[i];

	return lowest;
}

void BoxSelector::setAlignMode(int mode) {
	_align_mode = mode;
}

void BoxSelector::getAlignRect() {
	float screenx, screeny;
	QWidget *widget;
	GLint viewport[4];

	widget = RendererManager::getInstance()->getPreviewRenderer()->getRendererWidget();
	glGetIntegerv(GL_VIEWPORT, viewport);

	screenx = widget->mapFromGlobal(QCursor::pos()).x();
	screeny = viewport[3] - widget->mapFromGlobal(QCursor::pos()).y() - 1;

	glReadPixels(screenx,screeny,5,5,GL_DEPTH_COMPONENT,GL_FLOAT,_align_rect);
}

void BoxSelector::resizeAfterMouseMovement(int len) {
	double transz = ((double)len) * 0.01 *_edgelength[0]; 
	resize(transz);
}

void BoxSelector::setColors(float color[4], float selection_color[4]) {
	_color_normal[0] = _color_normal_transparent[0] = color[0];
	_color_normal[1] = _color_normal_transparent[0] = color[1];
	_color_normal[2] = _color_normal_transparent[0] = color[2];
	_color_normal[3] = 1.0f;
	_color_normal_transparent[0] = color[3];
	
	_color_selected[0] = selection_color[0];
	_color_selected[1] = selection_color[1];
	_color_selected[2] = selection_color[2];
	_color_selected[3] = 1.0f;
}

void BoxSelector::setSelectionIndex(int selection_index) { 
	_selection_index = selection_index; 
}


// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
