// Title:   LocalParticleSimulation.h
// Created: Thu Sep 25 15:16:39 2003
// Authors: Simon Heinzle, Tim Weyrich
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

/*
** DONE a copy if normal uints, in tmp
*/

#ifndef __LOCALPARTICLESIMULATION_H
#define __LOCALPARTICLESIMULATION_H

#ifdef _WIN32
	// turn off std::map warnings
	# pragma warning(push)
	# pragma warning(disable:4786) 
#endif

/**
 * Defines
 */
#define NO_OF_NEIGHBOURS_FOR_VARIATION 10
#define STEP_SIZE 0.1f


#include <vector>
#include "../../../../Core/Scene/src/Scene.h"
#include "../../../../Core/DataStructures/src/NeighbourHood.h"
#include "../../../../Core/DataStructures/src/QueryGrid.h"

#include "../../../../Utilities/MagicSoft/MgcEigen.h"
#include "../../../../Utilities/MagicSoft/MgcEigen.h"


#include "Action.h"
typedef unsigned int* uIntPtr;
typedef unsigned int  uInt;

class LocalParticleSimulation : public NormalAction {
	std::vector<unsigned int> *_surfel_indices;
	uIntPtr *_surfel_indices_uIntPtr;

	Vector3D *_positions_array;
	Vector3D *_normals_array;
	int _border_index;

	// shall we build a new kdtree every time we change the positions?
	Vector3D *_new_positions_array;
	// for verlet integration
	Vector3D *_old_positions_array;

	// query structures
	KdTree *_query_tree;
	//dynamicNeighbourHood<unsigned int> *_dynamic_hood;
	QueryGrid<uInt> *_dynamic_hood;

	// variation array
	float *_variation;
	float _variation_avg;
	

	// bounding box
	Vector3D min;
	Vector3D max;

	// parameters for repel
	int _border_k;
	int _max_neighbours;
	float _query_distance;
	int _num_of_iterations;
	float _force_scale;
	float _damping;

	// mls
	bool _full_mls;
	bool _use_mls;

	void repel(int max_neighbours, float query_distance, int num_of_iterations, float force_scale, float damping);
	void project();
	void getVariation(float variation_constant);
	void buildTree();
	void getBorder(int k_next);
	void initPositions();
	Vector3D doMLS(Vector3D source, int max_neighbours, float query_distance);
	void doAllMLS();
public:
	LocalParticleSimulation();
	~LocalParticleSimulation();
	void setParameters(int border_k, int max_neighbours, float query_distance, int num_of_iterations, float force_scale, float damping);
	void setMLS(bool on, bool full);
	void execute();
	UndoAction* executeUndo();
};



#ifdef _WIN32
	#pragma warning(pop)
#endif // _WIN32

#endif // __LOCALPARTICELSIMULATION

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
