// Title:   MemChecker.h
// Created: Thu Sep 25 15:16:32 2003
// Authors: Simon Heinzle, Tim Weyrich
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifndef __MEMCHECKER_H
#define __MEMCHECKER_H

#ifdef _WIN32
	// turn off std::map warnings
	# pragma warning(push)
	# pragma warning(disable:4786) // identifier truncated to 255 char
	# pragma warning(disable:4284) // return type ... (is not an udt)
#endif

#include <list>
#include <qapplication.h>

typedef struct {
	unsigned long	address;
	unsigned long	size;
	char			file[1024];
	unsigned long	line;
} sAllocInfo;

class MemChecker {
	std::list<sAllocInfo*> _list;
	MemChecker() { _list.clear(); }
public:
	static MemChecker* getInstance();
	void AddTrack(unsigned long addr, unsigned long size, const char *filename, unsigned long line_num);
	void RemoveTrack(unsigned long addr);
	void DumpUnfreed();
};

#ifdef _WIN32
	# pragma warning(pop)
#endif


#endif



 

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
