// Title:   OutlierClassificationDialogImpl.cpp
// Created: Thu Sep 25 15:16:17 2003
// Authors: Simon Heinzle, Tim Weyrich
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//



#include <qlineedit.h>
#include <qcheckbox.h>
#include <qfiledialog.h>
#include <qcombobox.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qradiobutton.h>
#include <qpushbutton.h>
#include <qstring.h>
#include <qfile.h>
#include <qbuttongroup.h>
#include <qstringlist.h>
#include <qcolordialog.h>
#include <qslider.h>
#include <qapplication.h>
#include <qlist.h>
#include <qstring.h>

#include <qdatetime.h>
#include <qstring.h>

#include "OutlierClassificationDialogImpl.h"
#include "Selector.h"
#include "ScanToolsDialogImpl.h"

#include "NewOverload.h"

#include <stdio.h>

#ifndef WIN32
#include <stdlib.h>
#define _gcvt gcvt
#endif

// ########################################
// singleton pattern, the singleton itself
// ########################################
OutlierClassificationDialogImpl *instance;
	

// #####################################
//  c'tor & d'tor (private / protected)
// #####################################

OutlierClassificationDialogImpl::OutlierClassificationDialogImpl (QWidget* parent, const char* name, WFlags flags) 
		: OutlierClassificationWindow(parent, name, flags)
{
	this->initDialog();
	w1 = 100.f/3.f;
	w2 = 100.f/3.f;
	w3 = 100.f/3.f;
	moving = false;
 }

OutlierClassificationDialogImpl::~OutlierClassificationDialogImpl(void)
{
}

// ####################################
//  public methods
// ####################################

OutlierClassificationDialogImpl *OutlierClassificationDialogImpl::getInstance(void) {
	if (instance == 0) {
		instance = new OutlierClassificationDialogImpl ();
		_MCnew( (unsigned long)instance, sizeof(instance), __FILE__, __LINE__);
	}
	return instance;
}

void OutlierClassificationDialogImpl::destroyInstance(void) {
	if (instance) {
		instance->close();
		_MCdelete((unsigned int)instance);
		delete instance;
	}
	instance = 0;
}

void OutlierClassificationDialogImpl::initDialog(void) {
	// create connections etc;
}



// ####################################
//  public slots
// ####################################


void OutlierClassificationDialogImpl::handleBuildButton() {
	QString string;
	QString string2;
	char buf[255];
	QTime timer;
	int k;

	timer.start();
	
	BuildLabel->setText("Building...");
	repaint();

	k = atoi(kNextEdit->text());
	ocl.setK(k);

	if (ocl.buildKNextGraph(-1)) {
	
		string  = "Status: graph built in ";
		sprintf(buf, "%d", timer.elapsed());
		string += buf;
		string += " msecs";
		
		MiniballBox->setEnabled(true);
		PlaneBox->setEnabled(true);
		BidirectionalBox->setEnabled(true);

	} else 
		string  = "Status: graph NOT built [error in buildKNextGraph()]";
	
	BuildLabel->setText(string + string2);
}

void OutlierClassificationDialogImpl::handleMBClassifyButton() {
	QString string;
	QString string2;
	QTime timer;

	timer.start();
	MBClassifyLabel->setText("Classifying...");
	repaint();
/*
	if (ocl.doMiniballClassification(ScanToolsDialogImpl::getInstance()->getSelectionIndex())) {
		string  = "Status: classified in ";
		sprintf(buf, "%d", timer.elapsed());
		string += buf;
		string += " msecs";
				
		string2  = "Cutoff (max = ";
		_gcvt(ocl.mbMaxC(), 10, buf);
		string2 += buf;
		string2 += ")";
		
		MBupperEdit->setText(buf);
		MBupperLabel->setText(string2);
		MBupperSlider->setValue(100);

		// update new combiner fields
		string2 = "Miniball, max = ";
		string2 += buf;
		CombinerMBBox->setTitle(string2);

		MBPreviewDeleteButton->setEnabled(true);
		MBDeleteButton->setEnabled(true);
		MBupperSlider->setEnabled(true);		
	} else 
		string = "Status: NOT classified [error in  doMiniballClassification]";
*/
	MBClassifyLabel->setText(string);
}

void OutlierClassificationDialogImpl::handlePFClassifyButton() {
	QString string;
	QString string2;
	QTime timer;

	timer.start();
	PFClassifyLabel->setText("Classifying...");
	repaint();
/*
	if (ocl.doPlaneClassification(ScanToolsDialogImpl::getInstance()->getSelectionIndex())) {
		string  = "Status: classified in ";
		sprintf(buf, "%d", timer.elapsed());
		string += buf;
		string += " msecs";
		
		PFPreviewDeleteButton->setEnabled(true);
		PFDeleteButton->setEnabled(true);
		PFupperSlider->setEnabled(true);

		string2  = "Cutoff (max = ";
		_gcvt(ocl.pfMaxC()*ocl.pfMaxConf(), 10, buf);
		string2 += buf;
		string2 += ")";

		PFupperEdit->setText(buf);
		PFupperLabel->setText(string2);
		PFupperSlider->setValue(100);

		// update new combiner field
		string2 = "Planefit, max = ";
		string2 += buf;
		CombinerPFBox->setTitle(string2);
	} else 
		string = "Status: NOT classified [error in  doMiniballClassification]";
*/
	PFClassifyLabel->setText(string);
}

void OutlierClassificationDialogImpl::handleBIClassifyButton() {
	QString string;
	QString string2;
	QTime timer;
	

	timer.start();
	BIClassifyLabel->setText("Classifying...");	
	repaint();
/*
	if (ocl.doBidirectionClassification(ScanToolsDialogImpl::getInstance()->getSelectionIndex())) {
		string  = "Status: classified in ";
		sprintf(buf, "%d", timer.elapsed());
		string += buf;
		string += " msecs";
		
		BIPreviewDeleteButton->setEnabled(true);
		BIDeleteButton->setEnabled(true);
		BIupperSlider->setEnabled(true);

		string2  = "Cutoff (max = ";
		_gcvt(ocl.biMaxC(), 10, buf);
		string2 += buf;
		string2 += ")";
		
		BIupperEdit->setText(buf);
		BIupperLabel->setText(string2);
		BIupperSlider->setValue(100);

		// update new combiner field
		string2 = "Bidirectional, max = ";
		string2 += buf;
		CombinerBIBox->setTitle(string2);
	} else 
		string = "Status: NOT classified [error in  doMiniballClassification]";
*/
	BIClassifyLabel->setText(string);
}


void OutlierClassificationDialogImpl::handleMBPreviewDeleteButton() {
	float margin;
	margin = atof(MBupperEdit->text());
	ocl.colorSurfelsCutoffMB(margin);
}

void OutlierClassificationDialogImpl::handleMBDeleteButton() {
	float margin;
	margin = atof(MBupperEdit->text());
	ocl.deleteSurfelsCutoffMB(margin);
	
	//disableButtonsAndBox();
}

void OutlierClassificationDialogImpl::handlePFPreviewDeleteButton() {
	float margin;
	margin = atof(PFupperEdit->text());
	ocl.colorSurfelsCutoffPF(margin);
} 

void OutlierClassificationDialogImpl::handlePFDeleteButton() {
	float margin;
	margin = atof(PFupperEdit->text());
	ocl.deleteSurfelsCutoffPF(margin);

	//disableButtonsAndBox();
}

void OutlierClassificationDialogImpl::handleBIPreviewDeleteButton() {
	float margin;
	margin = atof(BIupperEdit->text());
	ocl.colorSurfelsCutoffBI(margin);
}

void OutlierClassificationDialogImpl::handleBIDeleteButton() {
	float margin;
	margin = atof(BIupperEdit->text());
	ocl.deleteSurfelsCutoffBI(margin);

	//disableButtonsAndBox();
}

void OutlierClassificationDialogImpl::handleMBupperSlider() {
	float min, max;
	float ratio;
	float value;
	char buf[255];

	min = ocl.mbMinC();
	max = ocl.mbMaxC();
	
	// adjust upper value in edit field
	ratio = MBupperSlider->value();
	ratio /= 100.f;
	value = (max - min) * ratio + min;
	_gcvt(value, 10, buf);
	MBupperEdit->setText(buf);
}


void OutlierClassificationDialogImpl::handlePFupperSlider() {
	float min, max;
	float ratio;
	float value;
	char buf[255];

	min = ocl.pfMinC()*ocl.pfMinConf();
	max = ocl.pfMaxC()*ocl.pfMaxConf();
	
	// adjust upper value in edit field
	ratio = PFupperSlider->value();
	ratio /= 100.f;
	value = (max - min) * ratio + min;
	_gcvt(value, 10, buf);
	PFupperEdit->setText(buf);
}

void OutlierClassificationDialogImpl::handleBIupperSlider() {
	float min, max;
	float ratio;
	float value;
	char buf[255];

	min = ocl.biMinC();
	max = ocl.biMaxC();
	
	// adjust upper value in edit field
	ratio = BIupperSlider->value();
	ratio /= 100.f;
	value = (max - min) * ratio + min;
	_gcvt(value, 10, buf);
	BIupperEdit->setText(buf);
}

void OutlierClassificationDialogImpl::disableButtonsAndBox() {

	BuildLabel->setText("Status: needs to be rebuilt");

	MBPreviewDeleteButton->setEnabled(false);
	MBDeleteButton->setEnabled(false);
	MBupperSlider->setEnabled(false);
	MiniballBox->setEnabled(false);


	PFPreviewDeleteButton->setEnabled(false);
	PFDeleteButton->setEnabled(false);
	PFupperSlider->setEnabled(false);
	PlaneBox->setEnabled(false);

	BIPreviewDeleteButton->setEnabled(false);
	BIDeleteButton->setEnabled(false);
	BIupperSlider->setEnabled(false);
	BidirectionalBox->setEnabled(false);
}

void OutlierClassificationDialogImpl::handleALLClassifyButton() {
	QString string;
	QString string2;

	QTime timer;
	
	char buf[255];

	timer.start();
	ALLClassifyLabel->setText("Classifying...");	
	repaint();

	handleMBClassifyButton();
	handlePFClassifyButton();
	handleBIClassifyButton();

	string  = "Status: classified in ";
	sprintf(buf, "%d", timer.elapsed());
	string += buf;
	string += " msecs";
	
	ALLPreviewDeleteButton->setEnabled(true);
	ALLDeleteButton->setEnabled(true);

	ALLClassifyLabel->setText(string);
	
}

void OutlierClassificationDialogImpl::handleALLPreviewDeleteButton() {
	float weight_mb, weight_pf, weight_bi;
	float margin_mb, margin_pf, margin_bi;

	weight_mb = ((float)Method1Slider->value()) / 100.f - 0.01f;
	weight_pf = ((float)Method2Slider->value()) / 100.f - 0.01f;
	weight_bi = ((float)Method3Slider->value()) / 100.f - 0.01f;

	margin_mb = atof(MBupperEdit->text());
	margin_pf = atof(PFupperEdit->text());
	margin_bi = atof(BIupperEdit->text());
	
	ocl.colorSurfelsCutoffALL(weight_mb, margin_mb, weight_pf, margin_pf, weight_bi, margin_bi);
}

void OutlierClassificationDialogImpl::handleALLDeleteButton() {
	float weight_mb, weight_pf, weight_bi;
	float margin_mb, margin_pf, margin_bi;

	weight_mb = ((float)Method1Slider->value()) / 100.f - 0.01f;
	weight_pf = ((float)Method2Slider->value()) / 100.f - 0.01f;
	weight_bi = ((float)Method3Slider->value()) / 100.f - 0.01f;

	margin_mb = atof(MBupperEdit->text());
	margin_pf = atof(PFupperEdit->text());
	margin_bi = atof(BIupperEdit->text());
	
	ocl.deleteSurfelsCutoffALL(weight_mb, margin_mb, weight_pf, margin_pf, weight_bi, margin_bi);
}

void OutlierClassificationDialogImpl::handleMethod1Slider() {
	/*float r;
	float v;

	r = 100.f - ((float)Method1Slider->value());
	v = ((float)Method2Slider->value()) / (float) (Method3Slider->value() + Method2Slider->value());

	w1 = ((float) Method1Slider->value());
	w2 = v * r;
	w3 = (1.0f - v) * r;
	
	Method2Slider->setValue(w2);
	Method3Slider->setValue(w3);*/
 }

void OutlierClassificationDialogImpl::handleMethod2Slider() {
	/*float r;
	float v;

	r = 100.f - ((float)Method2Slider->value());
	v = ((float)Method1Slider->value()) / (float) (Method3Slider->value() + Method1Slider->value());

	w1 = v * r;
	w2 = ((float) Method2Slider->value());
	w3 = (1.0f - v) * r;
	
	Method2Slider->setValue(w2);
	Method1Slider->setValue(w1);*/
}

void OutlierClassificationDialogImpl::handleMethod3Slider() {
/*	float r;
	float v;

	r = 100.f - ((float)Method3Slider->value());
	v = ((float)Method1Slider->value()) / (float) (Method2Slider->value() + Method1Slider->value());

	w1 = v * r;
	w2 = (1.0f - v) * r;
	w3 = ((float) Method3Slider->value());
		
	Method1Slider->setValue(w1);
	Method3Slider->setValue(w3);*/
}

void OutlierClassificationDialogImpl::handleCombinerMethod1SliderMoved(int value) {
	float r;
	float v;

	w1 = ((float)CombinerMethod1Slider->value()) / 10000.0f;
	r = 100.f - w1;
	
	if (w2 + w3 != 0.0f)
		v = (w2) / ((w2) + (w3));
	else
		v = 0.5f;

	w2 = v * r;
	w3 = r - w2;
	
	CombinerMethod2Slider->setValue(w2*10000.f);
	CombinerMethod3Slider->setValue(w3*10000.f);
}

void OutlierClassificationDialogImpl::handleCombinerMethod2SliderMoved(int value) {
	float r;
	float v;

	w2 = ((float) CombinerMethod2Slider->value()) / 10000.f;
	r = 100.f - w2;
	
	if (w1 + w3 != 0.0f)
		v = (w1) / ((w1) + (w3));
	else
		v = 0.5f;

	w1 = v * r;
	w3 = r - w1;
	
	CombinerMethod3Slider->setValue(w3*10000.f);
	CombinerMethod1Slider->setValue(w1*10000.f);
}

void OutlierClassificationDialogImpl::handleCombinerMethod3SliderMoved(int value) {
	float r;
	float v;

	w3 = ((float) CombinerMethod3Slider->value()) / 10000.f;
	r = 100.f - w3;
	
	if (w1 + w2 != 0.0f)
		v = (w1) / ((w1) + (w2));
	else
		v = 0.5f;

	w1 = v * r;
	w2 = r - w1;
	
		
	CombinerMethod1Slider->setValue(w1*10000.f);
	CombinerMethod2Slider->setValue(w2*10000.f);
}

void OutlierClassificationDialogImpl::handleRestoreButton() {
	ocl.restoreColor();
}

void OutlierClassificationDialogImpl::passMousePress(QMouseEvent *me) { 
//	me->ignore();	
}

void OutlierClassificationDialogImpl::passMouseRelease(QMouseEvent *me){ 
//	me->ignore();
}

void OutlierClassificationDialogImpl::passMouseMove(QMouseEvent *me) { 
//	me->ignore();
}

void OutlierClassificationDialogImpl::passKeyPressEvent(QKeyEvent   *ke) {
	ke->ignore();
}

void OutlierClassificationDialogImpl::passKeyReleaseEvent(QKeyEvent   *ke) {
	ke->ignore();
}

void OutlierClassificationDialogImpl::setQCursor(QCursor *new_cursor) {
	_tool_cursor = new_cursor;
}

void OutlierClassificationDialogImpl::allPreview() {
	float weight_mb, weight_pf, weight_bi;
	float margin_mb, margin_pf, margin_bi;

	weight_mb = ((float)CombinerMethod1Slider->value()) / 100.f - 0.01f;
	weight_pf = ((float)CombinerMethod2Slider->value()) / 100.f - 0.01f;
	weight_bi = ((float)CombinerMethod3Slider->value()) / 100.f - 0.01f;

	margin_mb = atof(CombinerMBLineEdit->text());
	margin_pf = atof(CombinerPFLineEdit->text());
	margin_bi = atof(CombinerBILineEdit->text());
	
	ocl.colorSurfelsCutoffALL(weight_mb, margin_mb, weight_pf, margin_pf, weight_bi, margin_bi);
}

UndoAction* OutlierClassificationDialogImpl::allDelete() {
	float weight_mb, weight_pf, weight_bi;
	float margin_mb, margin_pf, margin_bi;

	weight_mb = ((float)CombinerMethod1Slider->value()) / 100.f - 0.01f;
	weight_pf = ((float)CombinerMethod2Slider->value()) / 100.f - 0.01f;
	weight_bi = ((float)CombinerMethod3Slider->value()) / 100.f - 0.01f;

	margin_mb = atof(CombinerMBLineEdit->text());
	margin_pf = atof(CombinerPFLineEdit->text());
	margin_bi = atof(CombinerBILineEdit->text());
	
	return ocl.deleteSurfelsCutoffALL(weight_mb, margin_mb, weight_pf, margin_pf, weight_bi, margin_bi);
}

void OutlierClassificationDialogImpl::classify() {
	QString string;
	QString string2;
	QTime timer;


	timer.start();
/*
	if (ocl.doMiniballClassification(ScanToolsDialogImpl::getInstance()->getSelectionIndex()) 
		&& ocl.doBidirectionClassification(ScanToolsDialogImpl::getInstance()->getSelectionIndex())
		&& ocl.doPlaneClassification(ScanToolsDialogImpl::getInstance()->getSelectionIndex())) {
		
		string  = "Combiner, classified in ";
		sprintf(buf, "%d", timer.elapsed());
		string += buf;
		string += " msecs";
		CombinerBox->setTitle(string);

		string2  = "Miniball, max = ";
		_gcvt(ocl.mbMaxC(), 10, buf);
		string2 += buf;
		string2 += ")";
		CombinerMBBox->setTitle(string2);
		
		string2  = "Planefit, max = ";
		_gcvt(ocl.pfMaxC(), 10, buf);
		string2 += buf;
		string2 += ")";
		CombinerPFBox->setTitle(string2);

		string2  = "Bidirectional, max = ";
		_gcvt(ocl.biMaxC(), 10, buf);
		string2 += buf;
		string2 += ")";
		CombinerBIBox->setTitle(string2);	
	} else
		CombinerBox->setTitle("Combinber - error in classification");
	*/
}


// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
