// Title:   ScanToolsDialogImpl.h
// Created: Thu Sep 25 15:15:32 2003
// Authors: Simon Heinzle, Tim Weyrich
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifndef __SCANTOOLSDIALOGIMPL_H_
#define __SCANTOOLSDIALOGIMPL_H_

#include <qlist.h>
#include <qstring.h>
#include <qbuttongroup.h>

#include "../../../../Renderers/OpenGLRenderer/src/OpenGLRenderer.h"

#include "../UI/ScanToolsWidget.h"
#include "OutlierClassificationDialogImpl.h"

/**
 * Dialog Class for ScanTools
 *
 * @author Simon Heinzle
 * @version 0.0
 */
class ScanToolsDialogImpl : public ScanToolsWidget {
	
    Q_OBJECT

public:

	/**
	 * This method creates an instance of this <code>ScanToolsDialogImpl</code> if necessary,
	 * and returns it. Only one instance at a time exists (singleton pattern).
	 *
	 * @return an instance of this <code>ScanToolsDialogImpl</code>
	 */
	static ScanToolsDialogImpl *getInstance (void);

	/**
	 * Destroys the instance of this <code>ScanToolsDialogImpl</code>, if there
	 * is one.
	 */
	static void destroyInstance();

	void passMousePress(QMouseEvent *me);
	void passMouseRelease(QMouseEvent *me);
	void passMouseMove(QMouseEvent *me);

	void setQCursor(QCursor *new_cursor);

signals:	
	void passKeyReleaseEvent(QKeyEvent *);
	void passKeyPressEvent(QKeyEvent *);

public slots:
	// key events
	void keyPressEvent(QKeyEvent *ke);
	void keyReleaseEvent(QKeyEvent *ke);


	

private:
	// singleton pattern, private c'tor
	ScanToolsDialogImpl (QWidget* parent = 0, const char* name = 0, WFlags flags = 0);
	void initDialog();
		
protected:	// To suppress warnings...
	virtual ~ScanToolsDialogImpl();

};

#endif  // __SCANTOOLSDIALOGIMPL_H_




// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
