// Title:   UndoRedoSpecialisations.cpp
// Created: Thu Sep 25 15:14:55 2003
// Authors: Simon Heinzle, Tim Weyrich
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

// Title:   UndoRedoSpecialisations.cpp
// Created: Thu Sep 25 15:14:54 2003
// Authors: Simon Heinzle, Tim Weyrich
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#include "UndoRedoSpecialisations.h"

#ifdef _WIN32
	// turn off std::map warnings
	# pragma warning(push)
	# pragma warning(disable:4786) 
#endif
	
#include "Selector.h"

#include "NewOverload.h"

/**
 * Undo & Redo Specialisation for DELETION / INSERTION of surfels.
 */
DeleteUndo::DeleteUndo(const std::vector<SurfelInterface*> &deleted_surfels, const std::map<unsigned int, Object*> &object_indices) {
	SurfelInterface *sfl;
	Scene *scene;
	std::vector<SurfelInterface*>::const_iterator it_src;
	
	scene = Scene::getInstance();
	// copy all deleted surfels
	_deleted_surfels.clear();
	for (it_src = deleted_surfels.begin(); it_src!=deleted_surfels.end(); it_src++) {
		sfl = _deleted_collection.addSurfel(*it_src,false);
		scene->setSurfelFlags(sfl, SurfelInterface::NO_FLAGS,false);
		_deleted_surfels.push_back(sfl);
	}
	
	_object_indices = object_indices;
	
}

DeleteUndo::DeleteUndo(const QVector<SurfelInterface> &deleted_surfels, const std::map<unsigned int, Object*> &object_indices) {
	SurfelInterface *sfl;
	Scene *scene;
	
	scene = Scene::getInstance();
	// copy all deleted surfels
	_deleted_surfels.clear();
	for (unsigned int i=0; i<deleted_surfels.count(); i++) {
		sfl = _deleted_collection.addSurfel(deleted_surfels.at(i), false);
		scene->setSurfelFlags(sfl,SurfelInterface::NO_FLAGS,false);
		
		_deleted_surfels.push_back(sfl);
	}
	
	_object_indices = object_indices;
}



	
DeleteUndo::~DeleteUndo() {
	_deleted_collection.clear();
	_deleted_surfels.clear();
	_object_indices.clear();
}



UndoAction* DeleteUndo::undo() {
	Selector *selector;
	std::vector<SurfelInterface*> inserted_surfels;
	std::vector<SurfelInterface*>::const_iterator it;
	std::map<unsigned int, Object*>::iterator obj_it;
	
	unsigned int i;

	selector = Selector::getInstance();
	obj_it = _object_indices.begin();
	// insert all deleted surfels.
	inserted_surfels.clear();
	for (it = _deleted_surfels.begin(), i = 0; it != _deleted_surfels.end(); it++, i++) {
		if ( (*obj_it).first < i)
			obj_it++;
		inserted_surfels.push_back(selector->insertSurfelToObject(*it,(*obj_it).second,false));
	}
	Scene::getInstance()->emitObjectModifiedSignal();
	Scene::getInstance()->emitRenderFrameSignal();

	selector->buildTree();
	UndoAction *undo = new InsertUndo(inserted_surfels, _object_indices);
	_MCnew( (unsigned long)undo, sizeof(undo), __FILE__, __LINE__);
	return undo;
}

InsertUndo::InsertUndo(const std::vector<SurfelInterface*> &inserted_surfels, const std::map<unsigned int, Object*> &object_indices) {
	_inserted_surfels = inserted_surfels;
	_object_indices = object_indices;
}

InsertUndo::~InsertUndo() {
}

UndoAction* InsertUndo::undo() {
	Selector *selector;
	std::vector<SurfelInterface*> new_deleted_surfels;
	std::vector<SurfelInterface*>::iterator it;
	std::map<unsigned int, Object*>::iterator obj_it;
	unsigned int i;

	selector = Selector::getInstance();
	obj_it = _object_indices.begin();
	// delete surfels.
	new_deleted_surfels.clear();
	for (it = _inserted_surfels.begin(), i = 0; it != _inserted_surfels.end(); it++, i++) {
		if ( (*obj_it).first < i)
			obj_it++;
		new_deleted_surfels.push_back(*it);
		//selector->deleteSurfelFromObject(*it,(*obj_it).second,false);
	}
	
	DeleteUndo *undo = new DeleteUndo(new_deleted_surfels, _object_indices);
	_MCnew( (unsigned long)undo, sizeof(undo), __FILE__, __LINE__);

	// delete surfels.
	new_deleted_surfels.clear();
	for (it = _inserted_surfels.begin(), i = 0; it != _inserted_surfels.end(); it++, i++) {
		if ( (*obj_it).first < i)
			obj_it++;
		//new_deleted_surfels.push_back(*it);
		selector->deleteSurfelFromObject(*it,(*obj_it).second,false);
	}


	Scene::getInstance()->emitObjectModifiedSignal();
	Scene::getInstance()->emitRenderFrameSignal();
	
	selector->buildTree();
	
/*
	DeleteUndo *undo = new DeleteUndo(new_deleted_surfels, _object_indices);
	_MCnew( (unsigned long)undo, sizeof(undo), __FILE__, __LINE__);
*/
	return undo;
}


/**
 * PositionNormalUndo
 */

PositionNormalUndo::PositionNormalUndo(const std::vector<SurfelInterface*>& surfels, const std::vector<Vector3D>& positions,const std::vector<Vector3D>& normals) {
	_surfels = surfels;
	_old_positions = positions;
	_old_normals = normals;
}


UndoAction *PositionNormalUndo::undo() {
	std::vector<Vector3D> normals;
	std::vector<Vector3D> positions;
	std::vector<SurfelInterface*>::const_iterator it;
	std::vector<Vector3D>::const_iterator n_it;
	std::vector<Vector3D>::const_iterator p_it;

	Scene *scene;

	scene = Scene::getInstance();
	n_it = _old_normals.begin();
	p_it = _old_positions.begin();

	for (it = _surfels.begin(); it != _surfels.end(); it++) {
		positions.push_back((*it)->getPosition());
		normals.push_back((*it)->getNormal());
		scene->setSurfelPosition(*it, *p_it, false);
		scene->setSurfelNormal(*it, *n_it, false);
		p_it++;
		n_it++;
	}

	scene->emitObjectModifiedSignal();
	scene->emitRenderFrameSignal();
	
	PositionNormalUndo *undo = new PositionNormalUndo(_surfels,positions, normals);
	_MCnew( (unsigned long)undo, sizeof(undo), __FILE__, __LINE__);

	return undo;
}

/**
 * FlagUndo
 */
FlagUndo::FlagUndo(const std::vector<SurfelInterface*> &surfels, const std::vector<int> &flags) {
	_surfels = surfels;
	_flags = flags;
}


void FlagUndo::addToThis(FlagUndo* undo) {
	_surfels.insert(_surfels.begin(), undo->_surfels.begin(), undo->_surfels.end());
	_flags.insert(_flags.begin(), undo->_flags.begin(), undo->_flags.end());;
}

UndoAction* FlagUndo::undo() {
	std::vector<int> new_flags;
	std::vector<SurfelInterface*>::const_iterator it;
	std::vector<int>::const_iterator f_it;

	Scene *scene;
	
	scene = Scene::getInstance();
	f_it = _flags.begin();
	it = _surfels.begin();

	for (it; it!=_surfels.end(); it++) {
		new_flags.push_back((*it)->getFlags());
		scene->setSurfelFlags(*it, (*f_it),false);
		f_it ++;
	}

	scene->emitObjectModifiedSignal();
	scene->emitRenderFrameSignal();
	
	FlagUndo *undo = new FlagUndo(_surfels, new_flags);
	_MCnew( (unsigned long)undo, sizeof(undo), __FILE__, __LINE__);
	return undo;
}

/**
 * ColorUndo
 */
ColorUndo::ColorUndo(const std::vector<SurfelInterface*> &surfels,	const std::vector<QRgb>& diffuse, const std::vector<QRgb>& specular) {
	_surfels = surfels;
	_old_diff = diffuse;
	_old_spec = specular;
}

UndoAction* ColorUndo::undo() {
	std::vector<QRgb> diffuse;
	std::vector<QRgb> specular;
	std::vector<SurfelInterface*>::const_iterator it;
	std::vector<QRgb>::const_iterator d_it;
	std::vector<QRgb>::const_iterator s_it;
	Scene *scene;

	scene = Scene::getInstance();
	it = _surfels.begin();
	d_it = _old_diff.begin();
	s_it = _old_spec.begin();

	for (it; it!=_surfels.end(); it++) {
		diffuse.push_back((*it)->getDiffuseColor());
		specular.push_back((*it)->getSpecularColor());
		scene->setSurfelDiffuseColor(*it, *d_it, false);
		scene->setSurfelSpecularColor(*it, *s_it, false);
		d_it++;
		s_it++;
	}

	scene->emitObjectModifiedSignal();
	scene->emitRenderFrameSignal();
	
	ColorUndo *undo = new ColorUndo(_surfels, diffuse, specular);
	_MCnew( (unsigned long)undo, sizeof(undo), __FILE__, __LINE__);

	return undo;
}





#ifdef _WIN32
//	#pragma warning(pop)
#endif // _WIN32
// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
